/*
 * Fragment.hpp
 *
 *  Created on: Aug 8, 2012
 *      Author: heber
 */

#ifndef FRAGMENT_HPP_
#define FRAGMENT_HPP_


// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <vector>

class Fragment {
public:
  typedef std::vector< std::vector<double> > positions_t;
  typedef std::vector< double > charges_t;
  typedef std::pair< std::vector<double>, double> nucleus_t;
  typedef std::vector< nucleus_t > nuclei_t;

  /** Default constructor of class Fragment.
   *
   */
  Fragment();

  /** Default constructor of class Fragment.
   *
   */
  Fragment(const nuclei_t &_nuclei) :
    nuclei(_nuclei)
  {}

  /** Constructor of class Fragment.
   *
   * @param _positions given positions
   * @param _charges given charges
   */
  Fragment(std::vector< std::vector<double> > &_positions, std::vector< double > &_charges);

  /** Adding another fragment onto this one.
   *
   * \note The operation is area-conserving, i.e. the new area is the sum of
   * both areas.
   *
   * @param other other fragment
   * @return ref to this instance
   */
  Fragment& operator+=(const Fragment &other);

  /** Assignment operator.
   *
   * @param other other fragment to make ourselves equal to
   * @return ref to this instance
   */
  Fragment& operator=(const Fragment &other);

  /** Subtracting another fragment from this one.
   *
   * @param other other fragment
   * @return ref to this instance
   */
  Fragment& operator-=(const Fragment &other);

private:
  /** Helper function that checks whether this nuclei is present.
   *
   * This operation is \f${\cal O}(n)\f$
   *
   * @param n nuclei to check
   * @return true - is contained, false - is not contained
   */
  bool containsNuclei(const nucleus_t &n) const;

  /** Seeks through all nuclei and removes matching one if found.
   *
   * @param n nuclei to remove
   */
  void removeNuclei(const nucleus_t &n);

private:
  nuclei_t nuclei;
};

template<typename T> T ZeroInstance();
template<> Fragment ZeroInstance<Fragment>();

#endif /* FRAGMENT_HPP_ */
